
'''
	description:
		This should be for calculating the land measures sum of 1 treasure,
		where that 1 treasure only has 1 package.
'''

'''
	import apoplast.shows.essential_nutrients.land.calculate_portions as calculate_portions
	calculate_portions.illustriously (
		land = land
	)
'''
	
from fractions import Fraction
	
def illustriously (
	land = {}
):
	grove = land ['grove']
	land_measures = land ['measures']
	
	for ingredient in grove:
		if (len (ingredient ["natures"]) == 0):
			continue;
		if (len (ingredient ["natures"]) >= 2):
			raise Exception (f"This def is for calculating the sum of a grove with only 1 treasure (food or supp) added")
		
		number_of_packages = Fraction (
			ingredient ["natures"] [ 0 ] ["amount"]
		)
		assert (number_of_packages == 1)

			
	return;





'''
	python3 insurance.py clouds/supp_NIH/nature/form/unit/status_tablet_261967.py
'''

import apoplast.clouds.supp_NIH.nature.form.unit as form_unit

def check_1 ():	
	unit = form_unit.calc (
		net_contents = [{
			"order": 1,
			"quantity": 90,
			"unit": "Tablet(s)",
			"display": "90 Tablet(s)"
		}],
		physical_state = {
			"langualCode": "E0155",
			"langualCodeDescription": "Tablet or Pill"
		},
		serving_sizes = [
			{
				"order": 1,
				"minQuantity": 3,
				"maxQuantity": 3,
				"minDailyServings": 1,
				"maxDailyServings": 1,
				"unit": "Tablet(s)",
				"notes": "",
				"inSFB": True
			}
		]
	)
	
	assert (unit == "Tablet"), unit
	
	return;
	
	
checks = {
	"check 1": check_1
}



'''
	python3 insurance.py shows/essential_nutrients/recipe/formulate/status_food/status_1.py
'''



import apoplast.insure.equality as equality

import apoplast.clouds.food_USDA.deliveries.one.assertions.foundational as assertions_foundational
import apoplast.clouds.food_USDA.examples as USDA_examples	
import apoplast.clouds.food_USDA.nature as food_USDA_nature

import apoplast.shows.essential_nutrients.recipe.formulate as formulate_recipe
import apoplast.shows.essential_nutrients.grove.seek_name_or_accepts as grove_seek_name_or_accepts

import json
from fractions import Fraction
from copy import deepcopy

def find_grams (measures):
	return Fraction (
		measures ["mass + mass equivalents"] ["per recipe"] ["grams"] ["fraction string"]
	)

def check_1 ():
	walnuts_1882785 = food_USDA_nature.create (
		USDA_examples.retrieve ("branded/walnuts_1882785.JSON")
	) ["essential nutrients"]
	vegan_pizza_2672996 = food_USDA_nature.create (
		USDA_examples.retrieve ("branded/vegan_pizza_2672996.JSON")
	) ["essential nutrients"]

	walnuts_1882785_1 = deepcopy (walnuts_1882785)
	vegan_pizza_2672996_1 = deepcopy (vegan_pizza_2672996)

	#print (json.dumps (walnuts_1882785 ["measures"], indent = 4))

	walnut_mass_and_mass_eq_grams = Fraction ("47746738154613173415299/112589990684262400000")
	walnut_energy_food_calories = Fraction ("154133/50")
	walnut_multiplier = 10

	vegan_pizza_2672996_mass_and_mass_eq_grams = Fraction ("1484714659522158138277/45035996273704960000")
	vegan_pizza_2672996_energy_food_calories = Fraction ("2627/20")
	vegan_pizza_2672996_multiplier = 100
	
	'''
		walnuts: 5021813355368299589/7036874417766400 713.6425999999999
		vegan pizza: 89531560592125469/2251799813685248 39.760000000000005
	'''	
	assert (
		walnuts_1882785 ["measures"] ==
		{
			"mass + mass equivalents": {
				"per recipe": {
					"grams": {
						"fraction string": str (walnut_mass_and_mass_eq_grams)
					}
				}
			},
			"energy": {
				"per recipe": {
					"food calories": {
						"fraction string": str (walnut_energy_food_calories)
					}
				}
			}
		}
	), walnuts_1882785 ["measures"]
	assert (
		vegan_pizza_2672996 ["measures"] ==
		{
			"mass + mass equivalents": {
				"per recipe": {
					"grams": {
						"fraction string": str (vegan_pizza_2672996_mass_and_mass_eq_grams)
					}
				}
			},
			"energy": {
				"per recipe": {
					"food calories": {
						"fraction string": str (vegan_pizza_2672996_energy_food_calories)
					}
				}
			}
		}
	), vegan_pizza_2672996 ["measures"]

	recipe = formulate_recipe.adroitly ([
		[ walnuts_1882785, walnut_multiplier ],
		[ vegan_pizza_2672996, vegan_pizza_2672996_multiplier ]
	])
	
	assert (
		recipe ["measures"] ==
		{
			"mass + mass equivalents": {
				"per recipe": {
					"grams": {
						"fraction string": str (
							(walnut_mass_and_mass_eq_grams * walnut_multiplier) +
							(vegan_pizza_2672996_mass_and_mass_eq_grams * vegan_pizza_2672996_multiplier)							
						)
					}
				}
			},
			"energy": {
				"per recipe": {
					"food calories": {
						"fraction string": str (
							(walnut_energy_food_calories * walnut_multiplier) +
							(vegan_pizza_2672996_energy_food_calories * vegan_pizza_2672996_multiplier)							
						)
					}
				}
			}
		}
	), walnuts_1882785 ["measures"]
	
	
	def assert_ingredient (ingredient):
		walnuts_1882785_ingredient_1 = grove_seek_name_or_accepts.politely (
			grove = walnuts_1882785_1 ["grove"],
			name_or_accepts = ingredient
		)
		vegan_pizza_2672996_ingredient_1 = grove_seek_name_or_accepts.politely (
			grove = vegan_pizza_2672996_1 ["grove"],
			name_or_accepts = ingredient
		)
		
		print ("original:")
		print (
			"	walnuts:", 
			(find_grams (walnuts_1882785_ingredient_1 ["measures"])),
			float (find_grams (walnuts_1882785_ingredient_1 ["measures"])),
		)
		print (
			"	vegan pizza:", 
			(find_grams (vegan_pizza_2672996_ingredient_1 ["measures"])),
			float (find_grams (vegan_pizza_2672996_ingredient_1 ["measures"]))
		)
		
		print ()
	
		walnuts_1882785_ingredient = grove_seek_name_or_accepts.politely (
			grove = walnuts_1882785 ["grove"],
			name_or_accepts = ingredient
		)
		vegan_pizza_2672996_ingredient = grove_seek_name_or_accepts.politely (
			grove = vegan_pizza_2672996 ["grove"],
			name_or_accepts = ingredient
		)
		recipe_ingredient = grove_seek_name_or_accepts.politely (
			grove = recipe ["grove"],
			name_or_accepts = ingredient
		)
		
		'''
			starts at:
				walnuts: 5021813355368299589/7036874417766400 713.6425999999999
				vegan pizza: 89531560592125469/2251799813685248 39.760000000000005
		'''	
		'''
			ends with:
				recipe: 7752113592587244929/11258999068426240 688.526
				walnuts: 456528486851663599/70368744177664 6487.66
				vegan pizza: 447657802960627345/562949953421312 795.2
				combined: 4099885697773936137/562949953421312 7282.86
		'''
		print ("recipe amounts:")
		print (
			"	recipe:", 
			find_grams (recipe_ingredient ["measures"]), 
			float (find_grams (recipe_ingredient ["measures"]))
		)
		print (
			"	walnuts:", 
			(find_grams (walnuts_1882785_ingredient ["measures"])),
			float (find_grams (walnuts_1882785_ingredient ["measures"])),
			
		)
		print (
			"	vegan pizza:", 
			(find_grams (vegan_pizza_2672996_ingredient ["measures"])),
			float (find_grams (vegan_pizza_2672996_ingredient ["measures"]))
		)
		print (
			"	combined:", 
			(
				(find_grams (walnuts_1882785_ingredient ["measures"])) +
				(find_grams (vegan_pizza_2672996_ingredient ["measures"]))
			),
			float (
				(find_grams (walnuts_1882785_ingredient ["measures"])) +
				(find_grams (vegan_pizza_2672996_ingredient ["measures"]))
			)
		)
		
		assert (
			find_grams (recipe_ingredient ["measures"]) == (
				(find_grams (walnuts_1882785_ingredient_1 ["measures"]) * walnut_multiplier) +
				(find_grams (vegan_pizza_2672996_ingredient_1 ["measures"]) * vegan_pizza_2672996_multiplier)
			)
		)
	
	
	assert_ingredient ("protein")
	assert_ingredient ("carbohydrates")
	assert_ingredient ("Cholesterol")
	assert_ingredient ("Sodium, Na")
	
checks = {
	"check 1": check_1
}



'''
	python3 insurance.py shows/essential_nutrients/recipe/formulate/status_food/status_0.py
'''



import apoplast.clouds.food_USDA.deliveries.one.assertions.foundational as assertions_foundational
import apoplast.clouds.food_USDA.examples as USDA_examples	
import apoplast.clouds.food_USDA.nature as food_USDA_nature
import apoplast.insure.equality as equality

import apoplast.shows.essential_nutrients.recipe.formulate as formulate_recipe
import apoplast.shows.essential_nutrients.grove.seek_name_or_accepts as grove_seek_name_or_accepts


from copy import deepcopy
from fractions import Fraction
import json



def check_1 ():
	walnuts_1882785 = food_USDA_nature.create (
		USDA_examples.retrieve ("branded/walnuts_1882785.JSON")
	) ["essential nutrients"]
	
	'''
		recipe_1: {
			"mass + mass equivalents": {
				"per recipe": {
					"grams": {
						#
						#	424.0762244 grams
						#						
						"fraction string": "47746738154613173415299/112589990684262400000"
					}
				}
			},
			"energy": {
				"per recipe": {
					"food calories": {
						#
						#	3082.66 food calories
						#
						"fraction string": "154133/50"
					}
				}
			}
		}
	'''
	recipe_1 = formulate_recipe.adroitly ([
		[ walnuts_1882785, 1 ]
	])
	assert (
		recipe_1 ["measures"] ==
		walnuts_1882785 [ "measures" ]
	)
	
	
	#print ("recipe_1:", json.dumps (walnuts_1882785 [ "measures" ], indent = 4))
	
	recipe_2 = formulate_recipe.adroitly ([
		[ walnuts_1882785, 2 ]
	])
	assert (
		recipe_2 ["measures"] ==
		{
			"mass + mass equivalents": {
				"per recipe": {
					"grams": {
						#
						#	424.0762244 grams	
						#		-> 848.1524488 = 47746738154613173415299/56294995342131200000
						#						
						"fraction string": "47746738154613173415299/56294995342131200000"
					}
				}
			},
			"energy": {
				"per recipe": {
					"food calories": {
						#
						#	3082.66 food calories
						#		-> 6165.32 = 154133/25
						#
						"fraction string": "154133/25"
					}
				}
			}
		}
	)
	
	
	
checks = {
	"check 1": check_1
}



'''
	import apoplast.shows.essential_nutrients.measures.merge as merge_measures
	merge_measures.calc (
		aggregate_measures,
		new_measures
	)
'''

from fractions import Fraction

def calc (
	aggregate_measures, 
	new_measures
):
	for measure in new_measures:
		pers = new_measures [ measure ]
		
		for per in pers:
			units = pers [per]
		
			if (per not in [ "per recipe" ]):
				raise Exception (f"The divisor found, '{ per }', was not accounted for.");
			
			for unit in units:
				#print ("unit:", unit)
			
				if (measure not in aggregate_measures):
					aggregate_measures [ measure ] = {}
					aggregate_measures [ measure ] [per] = {}
					aggregate_measures [ measure ] [per] [unit] = {}
					aggregate_measures [ measure ] [per] [unit] ["fraction string"] = "0"

			
				aggregate_measures [ measure ] [per] [unit] ["fraction string"] = str (
					Fraction (new_measures [ measure ] [per] [unit] ["fraction string"]) + 
					Fraction (aggregate_measures [ measure ] [per] [unit] ["fraction string"])
				)
				




'''
	import apoplast._ellipses as ellipses
	API_USDA_ellipse = ellipses.scan () ['USDA'] ['food']
	API_NIH_ellipse = ellipses.scan () ['NIH'] ['supplements']
'''

'''
/online ellipsis/cyte/ellipsis.json 

{
	"USDA": {
		"food": ""
	},
	"NIH": {
		"supplements": ""
	}
}
'''

import json
fp = open ("/online ellipsis/cyte/ellipsis.json", "r")
bounce = json.loads (fp.read ())
fp.close ()


def scan ():
	return bounce



'''
#
#	find name
#
import apoplast.shows.essential_nutrients.DB.scan.seek as seek_nutrient
nutrient = seek_nutrient.presently (
	for_each = lambda essential : True if "thiamin" in essential ["names"] else False
)
'''

'''
#
#	find region
#
import apoplast.shows.essential_nutrients.DB.scan.seek as seek_nutrient
nutrient = seek_nutrient.presently (
	for_each = lambda essential : True if essential ["region"] == 1 else False
)
'''

import apoplast.shows.essential_nutrients.DB.scan.list as essentials_list

def for_each ():
	return False
	
def presently (
	for_each = for_each
):
	essentials = essentials_list.retrieve ()
	for essential in essentials:
		if (for_each (essential)):
			return essential
	
	return None



'''
import apoplast.clouds.supp_NIH.nature.measured_ingredient as measured_ingredient_builder
'''

import json
import apoplast.measures._interpret.unit_kind as unit_kind

from fractions import Fraction

import apoplast.measures.energy.swap as energy_swap
import apoplast.measures.mass.swap as mass_swap
import apoplast.measures.number.decimal.reduce as reduce_decimal

def build (
	form,
	NIH_ingredient
):
	measured_ingredient = {
		"name": NIH_ingredient ["name"],
		"alternate names": NIH_ingredient ["alternateNames"],
		"measures": {},
		"unites": []
	}

	'''
		This parses and makes assertions about the NIH_ingredient 
	'''
	form_amount_per_package = form ["amount per package"]
	form_serving_size_amount = form ["serving size amount"]

	quantity_details = NIH_ingredient ["quantity"]
	assert (len (quantity_details) == 1);
	assert (
		str (quantity_details [0] ["servingSizeQuantity"]) == str (form_serving_size_amount)
	), (quantity_details [0] ["servingSizeQuantity"], form_serving_size_amount);
	
	unit = quantity_details [0] ["unit"]
	quantity = quantity_details [0]["quantity"]
	
	if (unit == "NP"):
		# The measurements are not known.
		return measured_ingredient
		
	kind = unit_kind.calc (unit)
	
	
	
	if (kind in [ "mass", "mass equivalent" ]):
		listed = (
			Fraction (quantity) *
			Fraction (form_serving_size_amount)
		)
		mass_plus_mass_eq_per_package_in_grams = Fraction (
			mass_swap.start (
				[ 
					listed, 
					unit 
				], 
				"grams",
				allow_equivalents = True
			)
		)
		
		if (kind == "mass equivalent"):
			is_equivalent = "yes"
		else:
			is_equivalent = "no"

		measured_ingredient ["measures"] ["mass + mass equivalents"] = {
			"per package": {
				"is equivalent": is_equivalent,
				"listed": [ 
					reduce_decimal.start (
						listed, 
						partial_size = 3
					), 
					unit 
				],
				"grams": {
					"decimal string": reduce_decimal.start (
						mass_plus_mass_eq_per_package_in_grams, 
						partial_size = 3
					),
					"fraction string": str (mass_plus_mass_eq_per_package_in_grams)
				}
			}
		}
		
	elif (kind == "energy"):
		listed = (
			Fraction (quantity) *
			Fraction (form_serving_size_amount)
		)
		energy_in_food_calories = Fraction (energy_swap.start ([ 
			listed, 
			unit 
		], "food calories"))
		
		measured_ingredient ["measures"] ["energy"] = {
			"per package": {
				"listed": [ 
					reduce_decimal.start (
						listed, 
						partial_size = 3
					), 
					unit 
				],
				"food calories": {
					"decimal string": reduce_decimal.start (
						energy_in_food_calories, 
						partial_size = 3
					),
					"fraction string": str (energy_in_food_calories)
				}				
			}
		}
	
	elif (kind == "biological activity"):
		assert (unit.lower () == "iu")
		
		listed = (
			Fraction (quantity) *
			Fraction (form_serving_size_amount)
		)
		
		measured_ingredient ["measures"] ["biological activity"] = {
			"per package": {
				"listed": [ 
					reduce_decimal.start (
						listed, 
						partial_size = 3
					), 
					unit 
				],
				"IU": {
					"decimal string": reduce_decimal.start (
						listed, 
						partial_size = 3
					),
					"fraction string": str (listed)
				}
			}
		}
	
	else:
		raise Exception (f"The kind '{ kind }' of the NIH ingredient was not accounted for.")
		
		
	return measured_ingredient



'''
	description:
		This should be for calculating the land measures sum of 1 treasure,
		however the treasure can have multiple packages.
'''


'''
	import apoplast.shows.essential_nutrients.land.measures.sums as land_measures_sums
	land_measures_sums.calc (
		land = land
	)
'''

'''
	import apoplast.shows.essential_nutrients.land.measures.sums as land_measures_sums
	import apoplast.shows.essential_nutrients.land.build as build_essential_nutrients_land
	land = build_essential_nutrients_land.eloquently ()
	land_measures_sums.calc (
		land = land
	)
'''


'''
	priorities:
		[ ] story 1 sums:
				
			each story 1 ingredient is added to these:
				land [
					"measures"
				][
					'mass + mass equivalents'
				][
					'per recipe'
				][
					'grams'
				][
					'fraction string'
				]
					
'''

from fractions import Fraction

def calc (
	land = {},
	records = 0
):
	grove = land ['grove']
	land_measures = land ['measures']
	
	for ingredient in grove:
		#print ("ingredient:", ingredient ["essential"] ["names"])
		
	
		#
		#	The number of packages
		#
		'''
			
		'''
		if (len (ingredient ["natures"]) == 0):
			continue;
		if (len (ingredient ["natures"]) >= 2):
			raise Exception (f"This def is for calculating the sum of a grove with only 1 treasure (food or supp) added")
		
		number_of_packages = Fraction (
			ingredient ["natures"] [ 0 ] ["amount"]
		)
		#assert (number_of_packages == 1), number_of_packages
	
		ingredient_measures = ingredient ["measures"]
		for measure_name in ingredient_measures:
			ingredient_measure = ingredient_measures [ measure_name ]
	
			if (measure_name in [ "biological activity" ]):
				continue;
	
			if (measure_name in [ "mass + mass equivalents", "energy" ]):
				if (measure_name == "mass + mass equivalents"):
					unit = "grams"
				elif (measure_name == "biological activity"):
					unit = "IU"
				elif (measure_name == "energy"):
					unit = "food calories"
				else:
					raise Exception ("?")
			else:
				raise Exception (f"Measure: '{ measure_name }' was not accounted for.") 
			
			addend = Fraction (
				ingredient_measure [ "per recipe" ] [ unit ] [ "fraction string" ]
			) * number_of_packages
			
			print ("addend:", float (addend))
			
			'''
				This adds the ingredient amount to the land amount.
			'''
			if (measure_name in land_measures):
				current_land_amount = Fraction (
					land_measures [ measure_name ]["per recipe"][unit]["fraction string"]
				)
				if (records >= 1):
					print ('"recipe" sums:')
					print ('	current:', current_land_amount)
					print ('	addend:', addend)
					print ('	addition:', current_land_amount + addend)
					print ()
				
				land_measures [ measure_name ] ["per recipe"] [ unit ] ["fraction string"] = str (
					current_land_amount +
					addend
				)
			else:
				raise Exception (f"The measure '{ measure_name }' was not accounted for.")
			
	return;